@@ -32,15 +32,21 @@ uint roll(){
   return d;
 }
 
-void diehard_craps(Test **test, int irun)
+int diehard_craps(Test **test, int irun)
 {
 
- uint i,j;
+ uint i;
  uint point,throw,tries,wins;
  double sum,p;
  Xtest ptest;
  Vtest vtest;
 
+ /*
+  * This is just for output display.
+  */
+ test[0]->ntuple = 0;
+ test[1]->ntuple = 0;
+
  /*
   * ptest.x = number of wins
   *   p = 244.0/495.0 is the probability of winning, so the mean
@@ -64,7 +70,7 @@ void diehard_craps(Test **test, int irun)
   * Allocate memory for Vtest struct vector (length 21) and initialize
   * it with the expected values.
   */
- Vtest_create(&vtest,21,"diehard_craps",gsl_rng_name(rng));
+ Vtest_create(&vtest,21);
  vtest.cutoff = 5.0;
  sum = 1.0/3.0;
  vtest.y[0] = sum;
@@ -150,5 +156,9 @@ void diehard_craps(Test **test, int irun)
    printf("# diehard_runs(): test[1]->pvalues[%u] = %10.5f\n",irun,test[1]->pvalues[irun]);
  }
 
+ Vtest_destroy(&vtest);
+
+ return(0);
+
 }
 
